''Name: Time Clock
''Author: Corbin Adkins
''Version: 5.8
''Mods: Previous versions proved complicated and with much errors and bugs due to complicated time-counting systems, so this
''one is entirely reformed into a more stable condition.
''Desription: [See additionally packaged files] 
CON

  _clkmode = xtal1 + pll16x            'Set the Xtal to 5Mhz @ high speed. 
  _xinfreq = 5_000_000

VAR

  byte mount                                          'Mount flag
  byte name[20]                                       'Buffer for the name of the person read off the SD card
  byte hour, minute, second, day, month, year, dow    'Time data
  
OBJ

  ser   : "FullDuplexSerial"                 'Serial object (for communication with the keypad)
  sd    : "fsrw"                             'SD card object
  rtc   : "DS1302_full"                      'RTC object
  sn    : "Simple_Numbers"                   'Object for calculating a string from the time data
  
PUB Start 

  ser.start(8,30,0,9600)                           'Start the serial driver for the keypad
  dira[9..11]~~                                    'Set the direction of the output pins
  outa[11]~~                                       'Output the safe light until the SD card is mounted
  sd.mount(3)
  mount := "!"
  outa[11]~
  rtc.init( 0, 1, 2 )                              'Start the RTC
  Main                                             'Go to the main
  
PUB Main | key 
  
  dira[16..23]~~
  outa[22..16]~
  repeat                     
    if ina[12] == 1                          'This checks for input on the "Remove" button
      if mount == "!"                        'If it's inserted, it closes and prepares to be ejected
        outa[11]~
        sd.pclose
        mount~
        outa[11]~~
      else                                   'If it's not inserted, it prepares to be inserted. 
        outa[11]~~
        'sd.mount(3)                         'This was removed for this version of the time clock
        mount := "!"
        outa[11]~
      waitcnt(clkfreq/2 + cnt) 
    key := ser.rxcheck                       'Get the key from the keypad
    if key == "#"                            'Check if the key is a pound sign
      outa[22..16] := %0000001               'Display the "wait" symbol
      Log                                    'Go to the log routine

PUB Log | key, code, bf, m, i, pin

  code := ser.rx                             'Get the next key from the keypad
  if code > "/" and code < ":"               'Check if the code is a number or not
    outa[22..16] := num[code-48]
  waitcnt(clkfreq/8 + cnt)
  key := ser.rx                              'Get the next keystroke
  sd.popen(string("users.txt"),"r")          'Get the users name from the "users.txt" file on the SD card
  repeat
    bf := sd.pgetc
    if bf == code
      bf := sd.pgetc
      repeat i from 0 to 20
        m := sd.pgetc
        if m > "/" and m < ":"
          quit
        else  
          name[i] := m
      name[i]~
      i~
      quit                                                               
  sd.pclose                                    'Close the file when the previous task is completed
  if sd.popen(string("hours.htm"),"r") == -1   'Check if the Hours file exists, and if it doesn't then create it and attach the header
    sd.popen(string("hours.htm"),"w")
    sd.sdstr(string("<h1 style=",34,"font-family:impact;font-color:red;",34,">This weeks hours</h1>"))
    sd.pclose
  if key == "C"                                'If the Clear key was pressed, cancel the operation and return to the main loop
    Main                                 
 ' if key == "A"
  sd.popen(string("hours.htm"),"a")            'Open up the Hours file to log the hours to the SD card in HTML                                                                    
  if key == "A"                                'See if they are checking in                                                                    
    sd.sdstr(string("<p style=",34,"color:green;",34,"><img src=",34,"user"))     'If in, green letters                                 
  else                                         'See if they are checking out                                                                    
    sd.sdstr(string("<p style=",34,"color:red;",34,"><img src=",34,"user"))       'If out, red letters                                 
  sd.pputc(code)                               'Insert the profile picture                                                                    
  sd.sdstr(string(".jpg",34," height=",34,"50",34," width=",34,"50",34," style=",34,"color:green;",34,"/>"))       
  sd.pputc(" ")                                                                                                    
  if key == "A"                                'Log that they checked in                                                                    
    sd.sdstr(@in)                                                                                                  
  else                                         'Log that they checked out                                                                    
    sd.sdstr(@out)                                                                                                 
  sd.pputc(" ")                                                                                                    
  sd.sdstr(@name)                              'Print the name                                                                    
  rtc.readTime(@hour,@minute,@second)          'Get the time                                                                    
  rtc.readDate(@day,@month,@year,@dow)         'Get the date                                                                    
  sd.sdstr(sn.decx(hour,2))                    'Write all the time data to the SD card                                                                    
  sd.pputc(":")                                                                                                    
  sd.sdstr(sn.decx(minute,2))                                                                                      
  sd.pputc(":")                                                                                                    
  sd.sdstr(sn.decx(second,2))                                                                                      
  sd.pputc(" ")                                                                                                    
  sd.sdstr(sn.decx(month,2))                                                                                       
  sd.pputc("/")                                                                                                    
  sd.sdstr(sn.decx(day,2))                                                                                         
  sd.sdstr(string("</p>"))                                                                                         
  'sd.pputc(" ")                                                                                                   
  sd.pclose                                    'Close the SD card's file                                                                    
  if key == "A"                                'Select which LED to light                                                                    
    pin := 9                                                                                                       
  else                                                                                                             
    pin := 10                                  'Set the direction and light the LED for 2 seconds                                                                    
  dira[pin]~~                                                                                                      
  outa[pin]~~                                                                                                      
  waitcnt(clkfreq*2 + cnt)                                                                                         
  outa[pin]~                                                                                                       
  Main                                                                                                             

DAT         

'Number data for the 7 segment display
num   byte    %1111110, %0110000, %1101101, %1111001, %0110011, %1011011, %1011111, %1110000, %1111111, %1111011
br    byte    "<br>",0            'Line break
out   byte    "OUT ",0            'Out line
in    byte    "IN ",0             'In line
